@@ -42,6 +42,12 @@
 #define EVT_EXPIRY_NSEC          ("expiry_nsec")
 #define EVT_MBUF_SZ              ("mbuf_sz")
 #define EVT_MAX_PKT_SZ           ("max_pkt_sz")
+#define EVT_PROD_ENQ_BURST_SZ    ("prod_enq_burst_sz")
+#define EVT_NB_ETH_QUEUES        ("nb_eth_queues")
+#define EVT_ENA_VECTOR           ("enable_vector")
+#define EVT_VECTOR_SZ            ("vector_size")
+#define EVT_VECTOR_TMO           ("vector_tmo_ns")
+#define EVT_PER_PORT_POOL	 ("per_port_pool")
 #define EVT_HELP                 ("help")
 
 void evt_options_default(struct evt_options *opt);
@@ -232,6 +238,22 @@ evt_dump_sched_type_list(struct evt_options *opt)
 	evt_dump_end;
 }
 
+static inline const char *
+evt_prod_id_to_name(enum evt_prod_type prod_type)
+{
+	switch (prod_type) {
+	default:
+	case EVT_PROD_TYPE_SYNT:
+		return "Synthetic producer lcores";
+	case EVT_PROD_TYPE_ETH_RX_ADPTR:
+		return "Ethdev Rx Adapter";
+	case EVT_PROD_TYPE_EVENT_TIMER_ADPTR:
+		return "Event timer adapter";
+	}
+
+	return "";
+}
+
 #define EVT_PROD_MAX_NAME_LEN 50
 static inline void
 evt_dump_producer_type(struct evt_options *opt)
