@@ -19,6 +19,7 @@
 
 void fill_actions(struct rte_flow_action *actions, uint64_t *flow_actions,
 	uint32_t counter, uint16_t next_table, uint16_t hairpinq,
-	uint64_t encap_data, uint64_t decap_data);
+	uint64_t encap_data, uint64_t decap_data, uint8_t core_idx,
+	bool unique_data, uint8_t rx_queues_count, uint16_t dst_port);
 
 #endif /* FLOW_PERF_ACTION_GEN */
