@@ -67,13 +67,13 @@ common_sources()
 
 linux_sources()
 {
-	find_sources "lib/librte_eal/linux" '*.[chS]'
+	find_sources "lib/eal/linux" '*.[chS]'
 	find_sources "kernel/linux" '*.[chS]'
 }
 
 bsd_sources()
 {
-	find_sources "lib/librte_eal/freebsd" '*.[chS]'
+	find_sources "lib/eal/freebsd" '*.[chS]'
 	find_sources "kernel/freebsd" '*.[chS]'
 }
 
@@ -85,14 +85,14 @@ arm_common()
 arm_32_sources()
 {
 	arm_common
-	find_sources "lib/librte_eal/arm" '*.[chS]' \
+	find_sources "lib/eal/arm" '*.[chS]' \
 					"$skip_64b_files"
 }
 
 arm_64_sources()
 {
 	arm_common
-	find_sources "lib/librte_eal/arm" '*.[chS]' \
+	find_sources "lib/eal/arm" '*.[chS]' \
 					 "$skip_32b_files"
 	find_sources "$source_dirs" '*arm64.[chS]'
 }
@@ -108,20 +108,20 @@ x86_common()
 x86_32_sources()
 {
 	x86_common
-	find_sources "lib/librte_eal/x86" '*.[chS]' \
+	find_sources "lib/eal/x86" '*.[chS]' \
 					"$skip_64b_files"
 }
 
 x86_64_sources()
 {
 	x86_common
-	find_sources "lib/librte_eal/x86" '*.[chS]' \
+	find_sources "lib/eal/x86" '*.[chS]' \
 					"$skip_32b_files"
 }
 
 ppc_64_sources()
 {
-	find_sources "lib/librte_eal/ppc" '*.[chS]'
+	find_sources "lib/eal/ppc" '*.[chS]'
 	find_sources "$source_dirs" '*altivec*.[chS]'
 }
 
