@@ -7,7 +7,7 @@
 #include <rte_bus_vdev.h>
 #include <rte_common.h>
 #include <rte_cryptodev.h>
-#include <rte_cryptodev_pmd.h>
+#include <cryptodev_pmd.h>
 #include <rte_pci.h>
 #include <rte_dev.h>
 #include <rte_malloc.h>
@@ -46,111 +46,6 @@ enum ccp_pmd_auth_opt {
 	CCP_PMD_AUTH_OPT_CPU,
 };
 
-/** parse integer from integer argument */
-static int
-parse_integer_arg(const char *key __rte_unused,
-		  const char *value, void *extra_args)
-{
-	int *i = (int *) extra_args;
-
-	*i = atoi(value);
-	if (*i < 0) {
-		CCP_LOG_ERR("Argument has to be positive.\n");
-		return -EINVAL;
-	}
-
-	return 0;
-}
-
-/** parse name argument */
-static int
-parse_name_arg(const char *key __rte_unused,
-	       const char *value, void *extra_args)
-{
-	struct rte_cryptodev_pmd_init_params *params = extra_args;
-
-	if (strlen(value) >= RTE_CRYPTODEV_NAME_MAX_LEN - 1) {
-		CCP_LOG_ERR("Invalid name %s, should be less than "
-			    "%u bytes.\n", value,
-			    RTE_CRYPTODEV_NAME_MAX_LEN - 1);
-		return -EINVAL;
-	}
-
-	strncpy(params->name, value, RTE_CRYPTODEV_NAME_MAX_LEN);
-
-	return 0;
-}
-
-/** parse authentication operation option */
-static int
-parse_auth_opt_arg(const char *key __rte_unused,
-		   const char *value, void *extra_args)
-{
-	struct ccp_pmd_init_params *params = extra_args;
-	int i;
-
-	i = atoi(value);
-	if (i < CCP_PMD_AUTH_OPT_CCP || i > CCP_PMD_AUTH_OPT_CPU) {
-		CCP_LOG_ERR("Invalid ccp pmd auth option. "
-			    "0->auth on CCP(default), "
-			    "1->auth on CPU\n");
-		return -EINVAL;
-	}
-	params->auth_opt = i;
-	return 0;
-}
-
-static int
-ccp_pmd_parse_input_args(struct ccp_pmd_init_params *params,
-			 const char *input_args)
-{
-	struct rte_kvargs *kvlist = NULL;
-	int ret = 0;
-
-	if (params == NULL)
-		return -EINVAL;
-
-	if (input_args) {
-		kvlist = rte_kvargs_parse(input_args,
-					  ccp_pmd_valid_params);
-		if (kvlist == NULL)
-			return -1;
-
-		ret = rte_kvargs_process(kvlist,
-					 CCP_CRYPTODEV_PARAM_MAX_NB_QP,
-					 &parse_integer_arg,
-					 &params->def_p.max_nb_queue_pairs);
-		if (ret < 0)
-			goto free_kvlist;
-
-		ret = rte_kvargs_process(kvlist,
-					 CCP_CRYPTODEV_PARAM_SOCKET_ID,
-					 &parse_integer_arg,
-					 &params->def_p.socket_id);
-		if (ret < 0)
-			goto free_kvlist;
-
-		ret = rte_kvargs_process(kvlist,
-					 CCP_CRYPTODEV_PARAM_NAME,
-					 &parse_name_arg,
-					 &params->def_p);
-		if (ret < 0)
-			goto free_kvlist;
-
-		ret = rte_kvargs_process(kvlist,
-					 CCP_CRYPTODEV_PARAM_AUTH_OPT,
-					 &parse_auth_opt_arg,
-					 params);
-		if (ret < 0)
-			goto free_kvlist;
-
-	}
-
-free_kvlist:
-	rte_kvargs_free(kvlist);
-	return ret;
-}
-
 static struct ccp_session *
 get_ccp_session(struct ccp_qp *qp, struct rte_crypto_op *op)
 {
@@ -299,26 +194,37 @@ static struct rte_pci_id ccp_pci_id[] = {
 
 /** Remove ccp pmd */
 static int
-cryptodev_ccp_remove(struct rte_vdev_device *dev)
+cryptodev_ccp_remove(struct rte_pci_device *pci_dev)
 {
-	const char *name;
+	char name[RTE_CRYPTODEV_NAME_MAX_LEN];
+	struct rte_cryptodev *dev;
+
+	if (pci_dev == NULL)
+		return -EINVAL;
+
+	rte_pci_device_name(&pci_dev->addr, name, sizeof(name));
+
+	if (name[0] == '\0')
+		return -EINVAL;
+
+	dev = rte_cryptodev_pmd_get_named_dev(name);
+	if (dev == NULL)
+		return -ENODEV;
 
 	ccp_pmd_init_done = 0;
-	name = rte_vdev_device_name(dev);
-	if (name == NULL)
-		return -EINVAL;
 
 	RTE_LOG(INFO, PMD, "Closing ccp device %s on numa socket %u\n",
 			name, rte_socket_id());
 
-	return 0;
+	return rte_cryptodev_pmd_destroy(dev);
 }
 
 /** Create crypto device */
 static int
 cryptodev_ccp_create(const char *name,
-		     struct rte_vdev_device *vdev,
-		     struct ccp_pmd_init_params *init_params)
+		     struct rte_pci_device *pci_dev,
+		     struct ccp_pmd_init_params *init_params,
+		     struct rte_pci_driver *pci_drv)
 {
 	struct rte_cryptodev *dev;
 	struct ccp_private *internals;
@@ -328,14 +234,14 @@ cryptodev_ccp_create(const char *name,
 			sizeof(init_params->def_p.name));
 
 	dev = rte_cryptodev_pmd_create(init_params->def_p.name,
-				       &vdev->device,
+				       &pci_dev->device,
 				       &init_params->def_p);
 	if (dev == NULL) {
 		CCP_LOG_ERR("failed to create cryptodev vdev");
 		goto init_error;
 	}
 
-	cryptodev_cnt = ccp_probe_devices(ccp_pci_id);
+	cryptodev_cnt = ccp_probe_devices(pci_dev, ccp_pci_id);
 
 	if (cryptodev_cnt == 0) {
 		CCP_LOG_ERR("failed to detect CCP crypto device");
@@ -343,6 +249,8 @@ cryptodev_ccp_create(const char *name,
 	}
 
 	CCP_LOG_DBG("CCP : Crypto device count = %d\n", cryptodev_cnt);
+	dev->device = &pci_dev->device;
+	dev->device->driver = &pci_drv->driver;
 	dev->driver_id = ccp_cryptodev_driver_id;
 
 	/* register rx/tx burst functions for data path */
@@ -361,22 +269,25 @@ cryptodev_ccp_create(const char *name,
 	internals->auth_opt = init_params->auth_opt;
 	internals->crypto_num_dev = cryptodev_cnt;
 
+	rte_cryptodev_pmd_probing_finish(dev);
+
 	return 0;
 
 init_error:
 	CCP_LOG_ERR("driver %s: %s() failed",
 		    init_params->def_p.name, __func__);
-	cryptodev_ccp_remove(vdev);
+	cryptodev_ccp_remove(pci_dev);
 
 	return -EFAULT;
 }
 
 /** Probe ccp pmd */
 static int
-cryptodev_ccp_probe(struct rte_vdev_device *vdev)
+cryptodev_ccp_probe(struct rte_pci_driver *pci_drv __rte_unused,
+	struct rte_pci_device *pci_dev)
 {
 	int rc = 0;
-	const char *name;
+	char name[RTE_CRYPTODEV_NAME_MAX_LEN];
 	struct ccp_pmd_init_params init_params = {
 		.def_p = {
 			"",
@@ -386,18 +297,15 @@ cryptodev_ccp_probe(struct rte_vdev_device *vdev)
 		},
 		.auth_opt = CCP_PMD_AUTH_OPT_CCP,
 	};
-	const char *input_args;
 
 	if (ccp_pmd_init_done) {
 		RTE_LOG(INFO, PMD, "CCP PMD already initialized\n");
 		return -EFAULT;
 	}
-	name = rte_vdev_device_name(vdev);
-	if (name == NULL)
+	rte_pci_device_name(&pci_dev->addr, name, sizeof(name));
+	if (name[0] == '\0')
 		return -EINVAL;
 
-	input_args = rte_vdev_device_args(vdev);
-	ccp_pmd_parse_input_args(&init_params, input_args);
 	init_params.def_p.max_nb_queue_pairs = CCP_PMD_MAX_QUEUE_PAIRS;
 
 	RTE_LOG(INFO, PMD, "Initialising %s on NUMA node %d\n", name,
@@ -407,21 +315,26 @@ cryptodev_ccp_probe(struct rte_vdev_device *vdev)
 	RTE_LOG(INFO, PMD, "Authentication offload to %s\n",
 		((init_params.auth_opt == 0) ? "CCP" : "CPU"));
 
-	rc = cryptodev_ccp_create(name, vdev, &init_params);
+	rte_pci_device_name(&pci_dev->addr, name, sizeof(name));
+
+	rc = cryptodev_ccp_create(name, pci_dev, &init_params, pci_drv);
 	if (rc)
 		return rc;
 	ccp_pmd_init_done = 1;
 	return 0;
 }
 
-static struct rte_vdev_driver cryptodev_ccp_pmd_drv = {
+static struct rte_pci_driver cryptodev_ccp_pmd_drv = {
+	.id_table = ccp_pci_id,
+	.drv_flags = RTE_PCI_DRV_NEED_MAPPING,
 	.probe = cryptodev_ccp_probe,
 	.remove = cryptodev_ccp_remove
 };
 
 static struct cryptodev_driver ccp_crypto_drv;
 
-RTE_PMD_REGISTER_VDEV(CRYPTODEV_NAME_CCP_PMD, cryptodev_ccp_pmd_drv);
+RTE_PMD_REGISTER_PCI(CRYPTODEV_NAME_CCP_PMD, cryptodev_ccp_pmd_drv);
+RTE_PMD_REGISTER_KMOD_DEP(CRYPTODEV_NAME_CCP_PMD, "* igb_uio | uio_pci_generic | vfio-pci");
 RTE_PMD_REGISTER_PARAM_STRING(CRYPTODEV_NAME_CCP_PMD,
 	"max_nb_queue_pairs=<int> "
 	"socket_id=<int> "
