@@ -5,10 +5,10 @@
 #ifndef _OTX2_CRYPTODEV_OPS_H_
 #define _OTX2_CRYPTODEV_OPS_H_
 
-#include <rte_cryptodev_pmd.h>
+#include <cryptodev_pmd.h>
 
-#define OTX2_CPT_MIN_HEADROOM_REQ	24
-#define OTX2_CPT_MIN_TAILROOM_REQ	8
+#define OTX2_CPT_MIN_HEADROOM_REQ	48
+#define OTX2_CPT_MIN_TAILROOM_REQ	208
 
 extern struct rte_cryptodev_ops otx2_cpt_ops;
 
