@@ -4,7 +4,7 @@
 #include <rte_common.h>
 #include <rte_hexdump.h>
 #include <rte_cryptodev.h>
-#include <rte_cryptodev_pmd.h>
+#include <cryptodev_pmd.h>
 #include <rte_bus_vdev.h>
 #include <rte_malloc.h>
 #include <rte_cpuflags.h>
@@ -230,6 +230,8 @@ cryptodev_scheduler_create(const char *name,
 		return -ENOMEM;
 	}
 
+	rte_cryptodev_pmd_probing_finish(dev);
+
 	return 0;
 }
 
