@@ -7,10 +7,7 @@
 
 #include <stdint.h>
 
-#include <rte_ethdev.h>
-
 #include "hns3_cmd.h"
-#include "hns3_ethdev.h"
 
 #define HNS3_ETHER_MAX_RATE		100000
 
@@ -211,6 +208,7 @@ int hns3_queue_to_tc_mapping(struct hns3_hw *hw, uint16_t nb_rx_q,
 			     uint16_t nb_tx_q);
 
 int hns3_update_queue_map_configure(struct hns3_adapter *hns);
-int hns3_dcb_port_shaper_cfg(struct hns3_hw *hw);
+int hns3_port_shaper_update(struct hns3_hw *hw, uint32_t speed);
+uint8_t hns3_txq_mapped_tc_get(struct hns3_hw *hw, uint16_t txq_no);
 
 #endif /* _HNS3_DCB_H_ */
