@@ -237,7 +237,6 @@ nfp_rtsym_size(const struct nfp_rtsym *sym)
 {
 	switch (sym->type) {
 	case NFP_RTSYM_TYPE_NONE:
-		printf("rtsym '%s': type NONE", sym->name);
 		return 0;
 	case NFP_RTSYM_TYPE_OBJECT:    /* Fall through */
 	case NFP_RTSYM_TYPE_FUNCTION:
@@ -245,7 +244,6 @@ nfp_rtsym_size(const struct nfp_rtsym *sym)
 	case NFP_RTSYM_TYPE_ABS:
 		return sizeof(uint64_t);
 	default:
-		printf("rtsym '%s': unknown type: %d", sym->name, sym->type);
 		return 0;
 	}
 }
@@ -259,11 +257,8 @@ nfp_rtsym_to_dest(struct nfp_cpp *cpp,
 		uint32_t *cpp_id,
 		uint64_t *addr)
 {
-	if (sym->type != NFP_RTSYM_TYPE_OBJECT) {
-		printf("rtsym '%s': direct access to non-object rtsym",
-				sym->name);
+	if (sym->type != NFP_RTSYM_TYPE_OBJECT)
 		return -EINVAL;
-	}
 
 	*addr = sym->addr + offset;
 
@@ -278,8 +273,6 @@ nfp_rtsym_to_dest(struct nfp_cpp *cpp,
 		*cpp_id = NFP_CPP_ISLAND_ID(NFP_CPP_TARGET_MU, action, token,
 				sym->domain);
 	} else {
-		printf("rtsym '%s': unhandled target encoding: %d",
-				sym->name, sym->target);
 		return -EINVAL;
 	}
 
@@ -298,10 +291,8 @@ nfp_rtsym_readl(struct nfp_cpp *cpp,
 	uint64_t addr;
 	uint32_t cpp_id;
 
-	if (offset + 4 > nfp_rtsym_size(sym)) {
-		printf("rtsym '%s': readl out of bounds", sym->name);
+	if (offset + 4 > nfp_rtsym_size(sym))
 		return -ENXIO;
-	}
 
 	ret = nfp_rtsym_to_dest(cpp, sym, action, token, offset, &cpp_id, &addr);
 	if (ret != 0)
@@ -322,10 +313,8 @@ nfp_rtsym_readq(struct nfp_cpp *cpp,
 	uint64_t addr;
 	uint32_t cpp_id;
 
-	if (offset + 8 > nfp_rtsym_size(sym)) {
-		printf("rtsym '%s': readq out of bounds", sym->name);
+	if (offset + 8 > nfp_rtsym_size(sym))
 		return -ENXIO;
-	}
 
 	if (sym->type == NFP_RTSYM_TYPE_ABS) {
 		*value = sym->addr;
@@ -365,6 +354,10 @@ nfp_rtsym_read_le(struct nfp_rtsym_table *rtbl, const char *name, int *error)
 		goto exit;
 	}
 
+#ifdef DEBUG
+	printf("Reading symbol %s with size %" PRIu64 " at %" PRIx64 "\n",
+		name, sym->size, sym->addr);
+#endif
 	switch (sym->size) {
 	case 4:
 		err = nfp_rtsym_readl(rtbl->cpp, sym, NFP_CPP_ACTION_RW, 0, 0, &val32);
@@ -413,10 +406,8 @@ nfp_rtsym_map(struct nfp_rtsym_table *rtbl, const char *name,
 
 	ret = nfp_rtsym_to_dest(rtbl->cpp, sym, NFP_CPP_ACTION_RW, 0, 0,
 			&cpp_id, &addr);
-	if (ret != 0) {
-		printf("rtsym '%s': mapping failed", name);
+	if (ret != 0)
 		return NULL;
-	}
 
 	if (sym->size < min_size) {
 		printf("Symbol %s too small (%" PRIu64 " < %u)\n", name,
