@@ -5,7 +5,7 @@
 #ifndef __THUNDERX_NICVF_ETHDEV_H__
 #define __THUNDERX_NICVF_ETHDEV_H__
 
-#include <rte_ethdev_driver.h>
+#include <ethdev_driver.h>
 
 #define THUNDERX_NICVF_PMD_VERSION      "2.0"
 #define THUNDERX_REG_BYTES		8
@@ -16,33 +16,32 @@
 #define NICVF_UNKNOWN_DUPLEX		0xff
 
 #define NICVF_RSS_OFFLOAD_PASS1 ( \
-	ETH_RSS_PORT | \
-	ETH_RSS_IPV4 | \
-	ETH_RSS_NONFRAG_IPV4_TCP | \
-	ETH_RSS_NONFRAG_IPV4_UDP | \
-	ETH_RSS_IPV6 | \
-	ETH_RSS_NONFRAG_IPV6_TCP | \
-	ETH_RSS_NONFRAG_IPV6_UDP)
+	RTE_ETH_RSS_PORT | \
+	RTE_ETH_RSS_IPV4 | \
+	RTE_ETH_RSS_NONFRAG_IPV4_TCP | \
+	RTE_ETH_RSS_NONFRAG_IPV4_UDP | \
+	RTE_ETH_RSS_IPV6 | \
+	RTE_ETH_RSS_NONFRAG_IPV6_TCP | \
+	RTE_ETH_RSS_NONFRAG_IPV6_UDP)
 
 #define NICVF_RSS_OFFLOAD_TUNNEL ( \
-	ETH_RSS_VXLAN | \
-	ETH_RSS_GENEVE | \
-	ETH_RSS_NVGRE)
+	RTE_ETH_RSS_VXLAN | \
+	RTE_ETH_RSS_GENEVE | \
+	RTE_ETH_RSS_NVGRE)
 
 #define NICVF_TX_OFFLOAD_CAPA ( \
-	DEV_TX_OFFLOAD_IPV4_CKSUM       | \
-	DEV_TX_OFFLOAD_UDP_CKSUM        | \
-	DEV_TX_OFFLOAD_TCP_CKSUM        | \
-	DEV_TX_OFFLOAD_OUTER_IPV4_CKSUM | \
-	DEV_TX_OFFLOAD_MBUF_FAST_FREE   | \
-	DEV_TX_OFFLOAD_MULTI_SEGS)
+	RTE_ETH_TX_OFFLOAD_IPV4_CKSUM       | \
+	RTE_ETH_TX_OFFLOAD_UDP_CKSUM        | \
+	RTE_ETH_TX_OFFLOAD_TCP_CKSUM        | \
+	RTE_ETH_TX_OFFLOAD_OUTER_IPV4_CKSUM | \
+	RTE_ETH_TX_OFFLOAD_MBUF_FAST_FREE   | \
+	RTE_ETH_TX_OFFLOAD_MULTI_SEGS)
 
 #define NICVF_RX_OFFLOAD_CAPA ( \
-	DEV_RX_OFFLOAD_CHECKSUM    | \
-	DEV_RX_OFFLOAD_VLAN_STRIP  | \
-	DEV_RX_OFFLOAD_JUMBO_FRAME | \
-	DEV_RX_OFFLOAD_SCATTER     | \
-	DEV_RX_OFFLOAD_RSS_HASH)
+	RTE_ETH_RX_OFFLOAD_CHECKSUM    | \
+	RTE_ETH_RX_OFFLOAD_VLAN_STRIP  | \
+	RTE_ETH_RX_OFFLOAD_SCATTER     | \
+	RTE_ETH_RX_OFFLOAD_RSS_HASH)
 
 #define NICVF_DEFAULT_RX_FREE_THRESH    224
 #define NICVF_DEFAULT_TX_FREE_THRESH    224
