@@ -9,7 +9,7 @@
 
 extern int mlx5_regex_logtype;
 
-#define MLX5_REGEX_LOG_PREFIX "regex_mlx5"
+#define MLX5_REGEX_LOG_PREFIX "mlx5_regex"
 /* Generic printf()-like logging macro with automatic line feed. */
 #define DRV_LOG(level, ...) \
 	PMD_DRV_LOG_(level, mlx5_regex_logtype, MLX5_REGEX_LOG_PREFIX, \
